@@ -82,9 +82,6 @@
 
 #include "qwaylandshellintegration_p.h"
 #include "qwaylandshellintegrationfactory_p.h"
-#include "qwaylandxdgshellintegration_p.h"
-#include "qwaylandwlshellintegration_p.h"
-#include "qwaylandxdgshellv6integration_p.h"
 
 #include "qwaylandinputdeviceintegration_p.h"
 #include "qwaylandinputdeviceintegrationfactory_p.h"
@@ -304,11 +301,14 @@ QPlatformTheme *QWaylandIntegration::createPlatformTheme(const QString &name) co
     return GenericWaylandTheme::createUnixTheme(name);
 }
 
+// May be called from non-GUI threads
 QWaylandClientBufferIntegration *QWaylandIntegration::clientBufferIntegration() const
 {
-    if (!mClientBufferIntegrationInitialized)
+    // Do an inexpensive check first to avoid locking whenever possible
+    if (Q_UNLIKELY(!mClientBufferIntegrationInitialized))
         const_cast<QWaylandIntegration *>(this)->initializeClientBufferIntegration();
 
+    Q_ASSERT(mClientBufferIntegrationInitialized);
     return mClientBufferIntegration && mClientBufferIntegration->isValid() ? mClientBufferIntegration.data() : nullptr;
 }
 
@@ -328,9 +328,12 @@ QWaylandShellIntegration *QWaylandIntegration::shellIntegration() const
     return mShellIntegration.data();
 }
 
+// May be called from non-GUI threads
 void QWaylandIntegration::initializeClientBufferIntegration()
 {
-    mClientBufferIntegrationInitialized = true;
+    QMutexLocker lock(&mClientBufferInitLock);
+    if (mClientBufferIntegrationInitialized)
+        return;
 
     QString targetKey;
     bool disableHardwareIntegration = qEnvironmentVariableIsSet("QT_WAYLAND_DISABLE_HW_INTEGRATION");
@@ -342,21 +345,26 @@ void QWaylandIntegration::initializeClientBufferIntegration()
         targetKey = QString::fromLocal8Bit(clientBufferIntegrationName);
     } else {
         targetKey = mDisplay->hardwareIntegration()->clientBufferIntegration();
+        if (targetKey == QLatin1String("wayland-eglstream-controller"))
+            targetKey = QLatin1String("wayland-egl");
     }
 
     if (targetKey.isEmpty()) {
         qWarning("Failed to determine what client buffer integration to use");
-        return;
+    } else {
+        QStringList keys = QWaylandClientBufferIntegrationFactory::keys();
+        if (keys.contains(targetKey)) {
+            mClientBufferIntegration.reset(QWaylandClientBufferIntegrationFactory::create(targetKey, QStringList()));
+        }
+        if (mClientBufferIntegration)
+            mClientBufferIntegration->initialize(mDisplay.data());
+        else
+            qWarning("Failed to load client buffer integration: %s\n", qPrintable(targetKey));
     }
 
-    QStringList keys = QWaylandClientBufferIntegrationFactory::keys();
-    if (keys.contains(targetKey)) {
-        mClientBufferIntegration.reset(QWaylandClientBufferIntegrationFactory::create(targetKey, QStringList()));
-    }
-    if (mClientBufferIntegration)
-        mClientBufferIntegration->initialize(mDisplay.data());
-    else
-        qWarning("Failed to load client buffer integration: %s\n", qPrintable(targetKey));
+    // This must be set last to make sure other threads don't use the
+    // integration before initialization is complete.
+    mClientBufferIntegrationInitialized = true;
 }
 
 void QWaylandIntegration::initializeServerBufferIntegration()
@@ -393,13 +401,14 @@ void QWaylandIntegration::initializeShellIntegration()
 {
     mShellIntegrationInitialized = true;
 
-    QByteArray integrationName = qgetenv("QT_WAYLAND_SHELL_INTEGRATION");
-    QString targetKey = QString::fromLocal8Bit(integrationName);
+    QByteArray integrationNames = qgetenv("QT_WAYLAND_SHELL_INTEGRATION");
+    QString targetKeys = QString::fromLocal8Bit(integrationNames);
 
     QStringList preferredShells;
-    if (!targetKey.isEmpty()) {
-        preferredShells << targetKey;
+    if (!targetKeys.isEmpty()) {
+        preferredShells = targetKeys.split(QLatin1Char(';'));
     } else {
+        preferredShells << QLatin1String("xdg-shell");
         preferredShells << QLatin1String("xdg-shell-v6");
         QString useXdgShell = QString::fromLocal8Bit(qgetenv("QT_WAYLAND_USE_XDG_SHELL"));
         if (!useXdgShell.isEmpty() && useXdgShell != QLatin1String("0")) {
@@ -418,9 +427,9 @@ void QWaylandIntegration::initializeShellIntegration()
         }
     }
 
-    if (!mShellIntegration || !mShellIntegration->initialize(mDisplay.data())) {
-        mShellIntegration.reset();
-        qWarning("Failed to load shell integration %s", qPrintable(targetKey));
+    if (!mShellIntegration) {
+        qCWarning(lcQpaWayland) << "Loading shell integration failed.";
+        qCWarning(lcQpaWayland) << "Attempted to load the following shells" << preferredShells;
     }
 }
 
@@ -452,15 +461,10 @@ void QWaylandIntegration::initializeInputDeviceIntegration()
 
 QWaylandShellIntegration *QWaylandIntegration::createShellIntegration(const QString &integrationName)
 {
-    if (integrationName == QLatin1Literal("wl-shell")) {
-        return QWaylandWlShellIntegration::create(mDisplay.data());
-    } else if (integrationName == QLatin1Literal("xdg-shell-v5")) {
-        return QWaylandXdgShellIntegration::create(mDisplay.data());
-    } else if (integrationName == QLatin1Literal("xdg-shell-v6")) {
-        return QWaylandXdgShellV6Integration::create(mDisplay.data());
-    } else if (QWaylandShellIntegrationFactory::keys().contains(integrationName)) {
-        return QWaylandShellIntegrationFactory::create(integrationName, QStringList());
+    if (QWaylandShellIntegrationFactory::keys().contains(integrationName)) {
+        return QWaylandShellIntegrationFactory::create(integrationName, mDisplay.data());
     } else {
+        qCWarning(lcQpaWayland) << "No shell integration named" << integrationName << "found";
         return nullptr;
     }
 }
